from pysc2.lib import actions, features, units
from pysc2.agents import base_agent

'''
Class to recover the current game state
'''

class GameState ():

    _SCV = 45
    _COMMAND_CENTER = 18
    _BARRACKS = 21
    _MARINE = 48
    _SUPPLY_DEPOT = 19

    # Declare Constants used to indicate unit alliances
    _PLAYER_FRIENDLY = 1
    _PLAYER_NEUTRAL = 3
    _PLAYER_HOSTILE = 4

    def __init__ (self, obs):
        self.stateDict = {
            "scvs": [],
            "idle scvs": [],
            "command centers": [],
            "marines": [],
            "barracks": [],
            "complete barracks": [],
            "supply depots": [],
            "minerals": 0,
            "can afford supply depot": False,
            "can afford barracks": False,
            "can afford marine": False,
            "can move marine": False,
            "command center location": "top-left"
            }

        self.updateState(obs)

    def updateState(self, obs):
        #Clear the lists that need to be cleared
        self.stateDict["scvs"].clear()
        self.stateDict["idle scvs"].clear()
        self.stateDict["marines"].clear()
        self.stateDict["command centers"].clear()
        self.stateDict["barracks"].clear()
        self.stateDict["supply depots"].clear()

        for unit in obs.observation.raw_units:
            if unit.alliance == self._PLAYER_FRIENDLY or \
                unit.alliance == self._PLAYER_NEUTRAL:
                #Only grabbing friendlies and neutral units
                if unit.unit_type == self._SCV:
                    self.stateDict["scvs"].append(unit)
                    if unit.order_length == 0: #Idle SCV
                        self.stateDict["idle scvs"].append(unit)
                elif unit.unit_type == self._COMMAND_CENTER:
                        self.stateDict["command centers"].append(unit)
                elif unit.unit_type == self._BARRACKS:
                    self.stateDict["barracks"].append(unit)
                    if unit.health == 1000: #Barracks is complete. I'm not clearing complete Barracks, becasue there could be a complete Barracks that gets damaged.
                        alreadyPresent = False
                        if len(self.stateDict["complete barracks"]) == 0:
                            self.stateDict["complete barracks"].append(unit)
                            alreadyPresent = True
                        else:
                            for barrack in self.stateDict["complete barracks"]:
                                if unit.tag == barrack.tag:
                                    alreadyPresent = True
                            if alreadyPresent == False:
                                self.stateDict["complete barracks"].append(unit)
                elif unit.unit_type == self._SUPPLY_DEPOT:
                    self.stateDict["supply depots"].append(unit)
                elif unit.unit_type == self._MARINE:
                    self.stateDict["marines"].append(unit)

        self.stateDict["minerals"] = obs.observation.player.minerals
        #Update can afford variables
        if self.stateDict["minerals"] > 150:
            self.stateDict["can afford barracks"] = True
        if self.stateDict["minerals"] > 100:
            self.stateDict["can afford supply depot"] = True
        if self.stateDict["minerals"] > 50 and len(self.stateDict["complete barracks"]) > 0:
            self.stateDict["can afford marine"] = True
        if len(self.stateDict["command centers"]) > 0:
            if self.stateDict["command centers"][0].y < (obs.observation.feature_screen.shape[2] / 2): 
                self.stateDict["command center location"] = "top-left"
            else:
                 self.stateDict["command center location"] = "bottom-right"
        if len(self.stateDict["marines"]) > 0:
            self.stateDict["can move marine"] = True

        return self.stateDict
    def getMarines (self):
        return self.stateDict["marines"]

    def getMinerals (self):
        return self.stateDict["minerals"]

    def canBuildSupplyDepot (self):
        return self.stateDict["can afford supply depot"]

    def canBuildBarracks (self):
        return self.stateDict["can afford barracks"]

    def canBuildMarine (self):
        return self.stateDict["can afford marine"]

    def getNumSupplyDepots (self):
        return len(self.stateDict["supply depots"])

    def getNumBarracks (self):
        return len(self.stateDict["barracks"])

    def canMoveMarine (self):
        return self.stateDict["can move marine"]

    def getCCLocation (self):
        return self.stateDict["command center location"]

    def getNumMarines (self):
        return len(self.stateDict["marines"])