from turtle import screensize
from SCAction import SCAction
from pysc2.lib import actions
import random as rand

'''
This class will be the core for AI training. 
We will start it with four options of where the marines can move. 
These four options will select all available marines and move the marines to the center of each corner of the map.
'''
class MoveMarine(SCAction):
    
    def __init__ (self, obs, friendDims, ccLocation, sector, marineTag):
        super(MoveMarine, self).__init__(obs, friendDims)
        self.ccLocation = ccLocation #Command Center location. Should be top-left or bottom-right as a string
        self.sector = sector
        self.marineTag = marineTag

    def getCCSector (self):
        return self.ccLocation

    '''
    The sector the marine moves to will be based on the map start config.
    The options for sector are:
        "home"
        "enemy"
        "retreat-horizontal"
        "retreat-vertical"
    marine: Can just be any marine. The Select All function 
    '''
    def getNextStep (self, obs, macroAction, screenSector):
        # Build the sector centers based on the map size. 
        screenSize = obs.observation.feature_screen.shape[2]
        centerSize = screenSize / 4
        centers = []
        centersX = []
        centersY = []
        marineAvailable = False

        for multiple in range(1,4):
            centers.append(centerSize * multiple)
        
        xScreenStart = int(centers[0])
        xScreenEnd = int(centers[2])
        multiple = int(xScreenStart / 2)
        for y_center in range(xScreenStart, xScreenEnd, multiple):
            centersY.append(y_center + multiple)
            
        multiple = int(screenSize / 4)
        for x_center in range(multiple, screenSize, multiple):
            centersX.append(x_center)

        for unit in obs.observation.raw_units:
            if unit.alliance == self._PLAYER_FRIENDLY and \
                unit.unit_type == self._MARINE:
                    marine = unit
                    marineAvailable = True
        if marineAvailable == True: #Make sure there is still a marine to select
            if self.ccLocation == "top-left":
                if self.sector == "enemy": #Send to bottom-right
                    x_coord = centersY[2]
                    y_coord = centersY[3]
                elif self.sector == "home":
                    x_coord = centersY[0]
                    y_coord = centersY[0]
                elif self.sector == "retreat-horizontal":
                    x_coord = centersX[0]
                    y_coord = centersY[3]
                elif self.sector == "retreat-vertical":
                    x_coord = centersY[2]
                    y_coord = centersY[0]

            elif self.ccLocation == "bottom-right":
                if self.sector == "enemy":
                    x_coord = 13
                    y_coord = 10
                elif self.sector == "home":
                    x_coord = centersY[2]
                    y_coord = centersY[3]
                elif self.sector == "retreat-horizontal":
                    x_coord = centersY[2]
                    y_coord = centersY[0]
                elif self.sector == "retreat-vertical":
                    x_coord = 13
                    y_coord = centersY[3]
            #Add variance so they don't all go to the same place
            varianceX = rand.uniform(-1, 1) * 4
            varianceY = rand.uniform(-1, 1) * 4
            return actions.RAW_FUNCTIONS.Attack_pt([0], self.marineTag, (x_coord + varianceX, y_coord + varianceY))
        else: #Marines have all died. Sad Face
            return actions.RAW_FUNCTIONS.no_op()





