'''
In order to limit the choices for the AI algorithm, this class creates sectors where the algorithm can move a unit.
Currently, the class will provide the center of each sector, but can provide options later for different parts of the sector
'''
import math

class ScreenSectors:

    def __init__ (self, map_size):
        self.map_size = map_size

        #This is for the scout action and indicates which corners have been visited
        self.sectorsVistied = {
            "top-left": False,
            "top-right": False,
            "bottom-left": False,
            "bottom-right": False}

    def createSectors (self, numSectorsX, numSectorsY):
        self.numSectorsX = numSectorsX
        self.numSectorsY = numSectorsY
        
        #Do some math to figure out the centers
        pxSectorX = math.floor(self.map_size/numSectorsX)
        pxSectorY = math.floor(self.map_size/numSectorsY) 
        
        #Determine how many sectors there will be
        numSectors = numSectorsX * numSectorsY

        self.sectorCenters = {}
        self.xSectorSize = math.floor(self.map_size / self.numSectorsX)
        self.ySectorSize = math.floor(self.map_size / self.numSectorsY)
        xCenterSize = math.floor(self.xSectorSize/2)
        yCenterSize = math.floor(self.ySectorSize/2)

        for sector in range(0, numSectors):
            xyPos = self.convertSector(sector)
            xyCenter = ((xyPos[0] * self.xSectorSize) + xCenterSize, (xyPos[1] * self.ySectorSize) + yCenterSize)
            self.sectorCenters[sector] = xyCenter

    def convertSector (self, sectorNum):
        yPos = (math.floor(sectorNum / self.numSectorsY)) * self.ySectorSize
        xPos = (sectorNum % self.numSectorsX) * self.xSectorSize

        return [xPos, yPos]

    def getSectorCenter (self, sectorNum):
        if sectorNum > self.numSectorsX * self.numSectorsY:
            return (0,0)
        else:
            return self.sectorCenters[sectorNum]
    
    def getCurrentSector (self, xPos, yPos):
        #I'm going to try and make this simple and just find the closest center point 
        closestSectorDistance = 999
        closestSector = 0
        sectorCounter = 0
        for sector in self.sectorCenters.items():
            distance = math.sqrt((xPos - sector[1][0])**2 + (yPos - sector[1][1])**2)
            if distance < closestSectorDistance:
                closestSectorDistance = distance
                closestSector = sectorCounter
            sectorCounter += 1
        return closestSector

    '''
    Helper function to determine which part of the map is friendly
    Command center is a raw_unit type
    '''
    def getFriendlyArea (self, commandCenter): 
        halfWay = self.map_size/2

        #Determine the X min and max
        if commandCenter.x <= halfWay: #Left Side
            minX = 0
            maxX = halfWay
        else:
            minX = halfWay
            maxX = self.map_size

        #Determine the Y min and max
        if commandCenter.y <= halfWay: #Left Side
            minY = 0
            maxY = halfWay
        else:
            minY = halfWay
            maxY = self.map_size

        return [minX, maxX, minY, maxX]
    